import cmd
import argparse
from time import sleep

from threading import Thread

import zmq
from vexmessage import decode_vex_message

from vexbot import __version__
from vexbot.adapters.messaging import ZmqMessaging
from vexbot.command_managers import CommandManager

from vexbot.commands.start_vexbot import start_vexbot as _start_vexbot
# from vexbot.commands.create_vexdir import create_vexdir
# from vexbot.commands.call_editor import call_editor


class Shell(cmd.Cmd):
    def __init__(self,
                 context=None,
                 prompt_name='vexbot',
                 publish_address=None,
                 subscribe_address=None,
                 **kwargs):

        super().__init__()
        self.messaging = ZmqMessaging('shell',
                                      publish_address,
                                      subscribe_address,
                                      'shell')

        self.command_manager = CommandManager(self.messaging)
        # FIXME
        self.command_manager._commands.pop('commands')
        self.stdout.write('Vexbot {}\n'.format(__version__))
        self.stdout.write("Type \"help\" for command line help or "
                          "\"commands\" for bot commands\n")
        if kwargs.get('already_running', False):
            self.stdout.write('vexbot already running\n')
        self.stdout.write('\n')

        self.command_manager.register_command('start_vexbot',
                                              _start_vexbot)

        self.messaging.start_messaging()

        self.prompt = prompt_name + ': '
        self.misc_header = "Commands"
        self._exit_loop = False

    def default(self, arg):
        if not self.command_manager.is_command(arg, call_command=True):
            self.messaging.send_command(command=arg)

    def run(self):
        frame = None
        while True and not self._exit_loop:
            try:
                # NOTE: not blocking here to check the _exit_loop condition
                frame = self.messaging.sub_socket.recv_multipart(zmq.NOBLOCK)
            except zmq.error.ZMQError:
                pass

            sleep(.5)

            if frame:
                message = decode_vex_message(frame)
                if message.type == 'RSP':
                    self.stdout.write("\n{}\n".format(self.doc_leader))
                    header = message.contents.get('original', 'Response')
                    contents = message.contents.get('response', None)
                    # FIXME
                    if (isinstance(header, (tuple, list))
                            and isinstance(contents, (tuple, list))
                            and contents):

                        for head, content in zip(header, contents):
                            self.print_topics(head, (contents,), 15, 70)
                    else:
                        if isinstance(contents, str):
                            contents = (contents,)
                        self.print_topics(header,
                                          contents,
                                          15,
                                          70)

                    self.stdout.write("vexbot: ")
                    self.stdout.flush()

                else:
                    # FIXME
                    print(message.type,
                          message.contents,
                          'fix me in shell adapter, run function')
                frame = None

    def _create_command_function(self, command):
        def resulting_function(arg):
            self.default(' '.join((command, arg)))
        return resulting_function

    def do_EOF(self, arg):
        self.stdout.write('\n')
        # NOTE: This ensures we exit out of the `run` method on EOF
        self._exit_loop = True
        return True

    def get_names(self):
        return dir(self)

    def do_help(self, arg):
        if arg:
            if self.command_manager.is_command(arg):
                doc = self.command_manager._commands[arg].__doc__
                if doc:
                    self.stdout.write("{}\n".format(str(doc)))
            else:
                self.messaging.send_command(command='help ' + arg)

        else:
            self.stdout.write("{}\n".format(self.doc_leader))
            # TODO: get these from robot?
            self.print_topics(self.misc_header,
                              ['start vexbot\nhelp [foo]', ],
                              15,
                              80)

    def add_completion(self, command):
        setattr(self,
                'do_{}'.format(command),
                self._create_command_function(command))

    """
    def _call_editor(self):
        vexdir = create_vexdir()
        code_output = call_editor(vexdir)
        try:
            code = compile(code_output, '<string>', 'exec')
        except Exception as e:
            print(e)

        local = {}
        exec(code, globals(), local)
        # need to add to commands?
        for k, v in local.items():
            if inspect.isfunction(v):
                self.command_manager.register_command(k, v)
    """


def _get_kwargs():
    parser = argparse.ArgumentParser()
    parser.add_argument('--publish_address', default=None)
    parser.add_argument('--subscribe_address', default=None)
    parser.add_argument('--prompt_name', default='vexbot')

    args = parser.parse_args()
    return vars(args)


def main(**kwargs):
    if not kwargs:
        kwargs = _get_kwargs()
    shell = Shell(**kwargs)
    cmd_loop_thread = Thread(target=shell.run)
    cmd_loop_thread.daemon = True
    cmd_loop_thread.start()

    shell.cmdloop()


if __name__ == '__main__':
    main()



import argparse as _argparse
import yaml as _yaml
from os import getenv as _getenv


class ArgEnvConfig:
    def __init__(self):
        self._environ = {}
        self._arg = _argparse.ArgumentParser()

    def initialize_argparse(self,
                            prog=None,
                            usage=None,
                            description=None,
                            epilog=None,
                            parents=[],
                            formatter_class=_argparse.HelpFormatter,
                            prefix_chars='-',
                            fromfile_prefix_chars=None,
                            argument_default=None,
                            conflict_handler='error',
                            add_help=True,
                            allow_abbrev=True):

        self._arg = _argparse.ArgumentParser(prog,
                                             usage,
                                             description,
                                             epilog,
                                             parents,
                                             formatter_class,
                                             prefix_chars,
                                             fromfile_prefix_chars,
                                             argument_default,
                                             conflict_handler,
                                             add_help,
                                             allow_abbrev)

    def add_argument(self, *args, **kwargs):
        try:
            environ = kwargs.pop('environ')
        except KeyError:
            environ = None

        argument = self._arg.add_argument(*args, **kwargs)

        if environ:
            self._environ[argument.dest] = environ

    def add_environment_variable(self, *args, **kwargs):
        pass

    def add_settings_file(self, *args, **kwargs):
        pass

    def get(self, value):
        args = self._arg.parse_args()
        result = getattr(args, value)
        if result is None:
            key = self._environ.get(value)
            if key:
                result = _getenv(key)
            else:
                result = None
        return result

    def get_args(self):
        return self._arg.parse_args()

    def load_settings(self, filepath):
        with open(filepath) as f:
            settings = _yaml.load(f)
        return settings
        settings = _yaml.load(filepath)



import logging

import pluginmanager

from vexmessage import decode_vex_message

from vexbot.messaging import Messaging
from vexbot.argenvconfig import ArgEnvConfig
from vexbot.command_managers import BotCommandManager
from vexbot.subprocess_manager import SubprocessManager


class Robot:
    def __init__(self, configuration, bot_name="vex"):
        # get the settings path and then load the settings from file
        settings_path = configuration.get('settings_path')
        settings = configuration.load_settings(settings_path)
        self.messaging = Messaging(settings)

        # create the plugin manager
        self.plugin_manager = pluginmanager.PluginInterface()
        # add the entry points of interest
        self.plugin_manager.add_entry_points(('vexbot.plugins',
                                              'vexbot.adapters'))

        # create the subprocess manager and add in the plugins
        self.subprocess_manager = SubprocessManager()
        self._update_plugins(settings,
                             self.subprocess_manager,
                             self.plugin_manager)

        subprocesses_to_start = settings.get('startup_adapters', [])
        subprocesses_to_start.extend(settings.get('startup_plugins', []))
        self.subprocess_manager.start(subprocesses_to_start)

        self.name = bot_name
        self._logger = logging.getLogger(__name__)
        self.command_manager = BotCommandManager(robot=self)
        try:
            import setproctitle
            setproctitle.setproctitle(bot_name)
        except ImportError:
            pass

    def run(self):
        while True:
            frame = self.messaging.subscription_socket.recv_multipart()
            msg = None
            try:
                msg = decode_vex_message(frame)
            except Exception:
                pass
            if msg:
                # Right now this is hardcoded into being only
                # the shell adapter
                # change this to some kind of auth code
                if ((msg.source == 'shell' or
                     msg.source == 'command_line') and msg.type == 'CMD'):

                    self.command_manager.parse_commands(msg)

    def _update_plugins(self,
                        settings,
                        subprocess_manager=None,
                        plugin_manager=None):
        """
        Helper process which loads the plugins from the entry points
        """
        if subprocess_manager is None:
            subprocess_manager = self.subprocess_manager
        if plugin_manager is None:
            plugin_manager = self.plugin_manager

        collect_ep = plugin_manager.collect_entry_point_plugins
        plugins, plugin_names = collect_ep()
        plugins = [plugin.__file__ for plugin in plugins]
        subprocess_manager.register(plugin_names, plugins)

        for name in plugin_names:
            try:
                plugin_settings = settings[name]
                values = []
                for k_v in plugin_settings.items():
                    values.extend(k_v)
            except KeyError:
                values = []
            self.subprocess_manager.update(name, setting=values)


def _get_config():
    config = ArgEnvConfig()
    config.add_argument('--settings_path',
                        default='settings.yml',
                        action='store')

    return config


if __name__ == '__main__':
    config = _get_config()
    robot = Robot(config)
    robot.run()



from os import path, mkdir


def create_vexdir():
    home_direct = path.expanduser("~")
    vexdir = path.join(home_direct, '.vexbot')
    if not path.isdir(vexdir):
        mkdir(vexdir)

    return vexdir



import atexit

from vexbot.commands.start_vexbot import start_vexbot
from vexbot.adapters.shell import main as shell_main


def _kill_vexbot(process):
    def inner():
        process.terminate()

    return inner


def main():
    settings, process = start_vexbot()
    if process and settings.get('kill_on_exit'):
        atexit.register(_kill_vexbot(process))

    shell_settings = settings['shell']
    if process is None:
        already_running = True
    else:
        already_running = False
    shell_settings['--already_running'] = already_running
    for key in set(shell_settings.keys()):
        value = shell_settings.pop(key)
        shell_settings[key[2:]] = value

    # Launch the shell interface
    shell_main(**shell_settings)


if __name__ == "__main__":
    main()



import os
import sys

from subprocess import Popen
from vexbot.commands.start_vexbot import _get_config


def restart_bot():
    """
    restarts vexbot
    """
    config = _get_config()
    settings_path = config.get('settings_path')
    directory = os.path.abspath(os.path.dirname(__file__))
    robot = os.path.abspath(os.path.join(directory, '..', 'robot.py'))
    s = "import time\nimport sys\nfrom subprocess import Popen\ntime.sleep(1" \
        ")\nPopen((sys.executable, '{}', '--settings_path', '{}'))"

    s = s.format(robot, settings_path)
    args = (sys.executable,
            '-c',
            s)

    Popen(args)
    sys.exit()



import string as _string
import collections as _collections

from vexmessage import Message as VexMessage

from vexbot.commands.restart_bot import restart_bot as _restart_bot


_INDENTCHARS = _string.ascii_letters + _string.digits + '_'


def _msg_list_wrapper(func: _collections.Callable):
    """
    wraps methods that don't take the `msg` type and instead take a list of
    args
    """
    def inner(msg: VexMessage):
        parsed = msg.contents.get('parsed_args')
        if parsed:
            return func(parsed)
        _, arg = _get_command_and_args(msg)
        commands = arg.split()

        return func(commands)

    inner.__doc__ = func.__doc__
    return inner


def _get_command_and_args(message: (str, VexMessage)) -> (str, str):
    if isinstance(message, VexMessage):
        command = message.contents.get('command', None)
    else:
        command = message

    if command is None:
        return None, None

    command = command.strip()
    i, n = 0, len(command)
    while i < n and command[i] in _INDENTCHARS:
        i = i + 1
    command, arg = command[:i], command[i:].strip()
    return command, arg


class CommandManager:
    def __init__(self, messaging):
        # NOTE: commands is a dict of dicts and there is nested parsing
        self._commands = {}
        self._messaging = messaging
        self._commands['help'] = _msg_list_wrapper(self._help)
        self._commands['commands'] = self._cmd_commands

    def register_command(self,
                         command: str,
                         function_or_dict: (_collections.Callable, dict)):
        """
        `func_or_nested` can either be a function or a dictionary
        """
        self._commands[command] = function_or_dict

    def is_command(self, command: str, call_command: bool=False) -> bool:
        command, arg = _get_command_and_args(command)
        callback, command, args = self._get_callback_recursively(command, arg)
        if callback and call_command:
            try:
                callback(args)
            except TypeError:
                callback()
            return True
        else:
            return bool(callback)

    def _get_callback_recursively(self,
                                  command: str,
                                  args: (list,
                                         str)=None) -> (_collections.Callable,
                                                        str,
                                                        list):
        """
        returns callback, command string, and args
        """

        if not command:
            return None, None, None
        if isinstance(args, str):
            args = args.split()
        elif args is None:
            args = []

        callback = self._commands.get(command)

        if isinstance(callback, _collections.Callable):
            return callback, command, args
        elif isinstance(callback, dict):
            dict_ = callback
        elif callback is None:
            return None, None, None
        else:
            s = '{} is not a callable function or a dict'
            raise TypeError(s.format(command))

        if not args:
            return None, None, None
        commands = []
        commands.append(command)

        # NOTE: Can't iterate None
        for command_number, command in enumerate(args):
            callback = dict_.get(command)
            if callback is None:
                return None, None, None

            commands.append(command)
            # dynamically reassigns the `dict_` value to travese nested dicts
            if isinstance(callback, dict):
                dict_ = callback
            elif isinstance(callback, _collections.Callable):
                break
            else:
                s = '{} is not a callable function or a dict'
                raise TypeError(s.format(callback.__name__))

        command_number += 1

        command_str = ' '.join(commands)
        return callback, command_str, args[command_number:]

    def parse_commands(self, msg: VexMessage):
        command, arg = _get_command_and_args(msg)

        if not command:
            return

        callback, command, args = self._get_callback_recursively(command, arg)
        msg.contents['parsed_args'] = args

        if callback:
            try:
                results = callback(msg)
            except TypeError:
                results = callback()

            if results:
                self._messaging.send_response(target=msg.source,
                                              original=command,
                                              response=results)

    def message_wrapper(self, func: _collections.Callable):
        """
        wraps a function and passes the messaging object as the first
        argument to the wrapped function
        """
        def inner(*args, **kwargs):
            return func(self._messaging, *args, **kwargs)
        # Fix the function metadata
        inner.__doc__ = func.__doc__
        inner.__str__ = func.__str__
        inner.__repr__ = func.__repr__

        return inner

    def _cmd_commands(self, msg: VexMessage):
        """
        returns a list of all available commands
        works recursively
        """
        def get_commands(d: dict):
            """
            recursive command
            """
            commands = []
            if not isinstance(d, dict):
                return commands

            for k, v in d.items():
                if isinstance(v, dict):
                    # NOTE: recursive command
                    stuff = get_commands(v)
                    for s in stuff:
                        commands.append(k + ' ' + s)
                else:
                    commands.append(k)
            return commands

        return get_commands(self._commands)

    def _send_command_not_found(self, target: str, original: str):
        """
        helper method
        """
        self._messaging.send_response(target=target,
                                      response='Command not found',
                                      original=original)

    def _help(self, args):
        if not args:
            return self._commands()
        else:
            docs = []
            for arg in args:
                doc = self._commands.get(arg, None).__doc__
                if doc:
                    docs.append(doc)

            if docs:
                return docs


class BotCommandManager(CommandManager):
    def __init__(self, robot):
        super().__init__(robot.messaging)
        # nested command dict
        subprocess = {}

        # alias for pep8
        s_manager = robot.subprocess_manager
        subprocess['settings'] = _msg_list_wrapper(s_manager.settings)

        self._commands['subprocess'] = subprocess

        self._commands['killall'] = s_manager.killall
        self._commands['restart_bot'] = _restart_bot
        self._commands['alive'] = self._alive

        self._commands['start'] = _msg_list_wrapper(s_manager.start)
        self._commands['subprocesses'] = s_manager.registered_subprocesses
        self._commands['restart'] = _msg_list_wrapper(s_manager.restart)
        self._commands['kill'] = _msg_list_wrapper(s_manager.kill)
        self._commands['terminate'] = _msg_list_wrapper(s_manager.terminate)
        self._commands['running'] = s_manager.running_subprocesses

    def _alive(self, msg):
        """
        queries the subprocesses to check their status using messaging.

        can also use the `running` command to see what's running locally
        """
        # FIXME
        values = list(self._commands['subprocesses']())
        if values:
            try:
                values.remove(msg.source)
            except ValueError:
                pass
            for v in values:
                self._messaging.send_command(target=v, command='alive')


class AdapterCommandManager(CommandManager):
    def __init__(self, messaging):
        super().__init__(messaging)
        self._commands['alive'] = self._alive

    def _alive(self, *args):
        self._messaging.send_status('CONNECTED')



import logging
import argparse
import signal
import atexit
from threading import Thread

import zmq

from sleekxmpp import ClientXMPP
from sleekxmpp.exceptions import IqError, IqTimeout
from time import sleep

from vexmessage import decode_vex_message

from vexbot.command_managers import AdapterCommandManager
from vexbot.adapters.messaging import ZmqMessaging # flake8: noqa


class ReadOnlyXMPPBot(ClientXMPP):
    def __init__(self,
                 jid,
                 password,
                 room,
                 publish_address,
                 subscribe_address,
                 service_name,
                 bot_nick='EchoBot',
                 **kwargs):

        # Initialize the parent class
        super().__init__(jid, password)
        self.messaging = ZmqMessaging(service_name,
                                      publish_address,
                                      subscribe_address,
                                      'xmpp')

        self.messaging.start_messaging()
        self.command_manager = AdapterCommandManager(self.messaging)

        self.room = room
        self.nick = bot_nick
        self.log = logging.getLogger(__file__)

        # One-shot helper method used to register all the plugins
        self._register_plugin_helper()

        self.add_event_handler("session_start", self.start)
        self.add_event_handler("groupchat_message", self.muc_message)
        self.add_event_handler('connected', self._connected)
        self.add_event_handler('disconnected', self._disconnected)
        self._thread = Thread(target=self.run)
        self._thread.daemon = True
        self._thread.start()

    def run(self):
        while True:
            frame = self.messaging.sub_socket.recv_multipart()
            message = decode_vex_message(frame)
            if message.type == 'CMD':
                self.command_manager.parse_commands(message)

    def _disconnected(self, *args):
        self.messaging.send_status('DISCONNECTED')

    def _connected(self, *args):
        self.messaging.send_status('CONNECTED')

    def _register_plugin_helper(self):
        """
        One-shot helper method used to register all the plugins
        """
        # Service Discovery
        self.register_plugin('xep_0030')
        # XMPP Ping
        self.register_plugin('xep_0199')
        # Multiple User Chatroom
        self.register_plugin('xep_0045')

    def start(self, event):
        self.log.info('starting xmpp')
        self.send_presence()
        self.plugin['xep_0045'].joinMUC(self.room,
                                        self.nick,
                                        wait=True)

        self.get_roster()

    def muc_message(self, msg):
        self.messaging.send_message(author=msg['mucnick'], message=msg['body'])


def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--local', help='local arg for string parsing')
    parser.add_argument('--domain', help='domain for xmpp')
    parser.add_argument('--room', help='room!')
    parser.add_argument('--resource', help='resource')
    parser.add_argument('--password', help='password')
    parser.add_argument('--service_name')
    parser.add_argument('--publish_address')
    parser.add_argument('--subscribe_address')
    parser.add_argument('--bot_nick')

    return parser.parse_args()


def _handle_close(messaging):
    def inner(*args):
        _send_disconnect(messaging)()
    return inner


def _send_disconnect(messaging):
    def inner():
        messaging.send_status('DISCONNECTED')
    return inner


def main():
    args = _get_args()
    jid = '{}@{}/{}'.format(args.local, args.domain, args.resource)
    kwargs = vars(args)
    already_running = False

    try:
        xmpp_bot = ReadOnlyXMPPBot(jid, **kwargs)
    except zmq.ZMQError:
        already_running = True

    if not already_running:
        messaging = xmpp_bot.messaging
        atexit.register(_send_disconnect(messaging))
        handle_close = _handle_close(messaging)
        signal.signal(signal.SIGINT, handle_close)
        signal.signal(signal.SIGTERM, handle_close)

        while True:
            try:
                xmpp_bot.connect()
                xmpp_bot.process(block=True)
            except Exception as e:
                xmpp_bot.log.error(e)

if __name__ == '__main__':
    main()



import sys
from os import path
from subprocess import Popen
import zmq
from vexbot.argenvconfig import ArgEnvConfig


def _get_config():
    config_manager = ArgEnvConfig()
    config_manager.add_argument('--settings_path',
                                environ='VEXBOT_SETTINGS',
                                action='store')

    return config_manager


def _running(address_to_check):
    context = zmq.Context()
    socket = context.socket(zmq.PUB)
    already_running = False
    try:
        socket.bind(address_to_check)
    except zmq.ZMQError:
        already_running = True

    if not already_running:
        socket.disconnect(address_to_check)

    return already_running


def start_vexbot():
    """
    starts up an instance of vexbot if one isn't running currently
    """
    config = _get_config()
    settings_path = config.get('settings_path')
    settings = config.load_settings(settings_path)
    process = None

    if not _running(settings.get('monitor_address')):
        root_directory = path.abspath(path.join(path.dirname(__file__), '..'))
        robot_filepath = path.join(root_directory, 'robot.py')

        # Start the subprocess
        main_robot_args = (sys.executable,
                           robot_filepath,
                           '--settings_path',
                           settings_path)

        process = Popen(main_robot_args)

    return settings, process



import sys
import asyncio
import argparse
import atexit
import signal

import irc3
import zmq
from zmq import ZMQError
from irc3.plugins.autojoins import AutoJoins
from vexmessage import decode_vex_message

from vexbot.command_managers import AdapterCommandManager
from vexbot.adapters.messaging import ZmqMessaging


@irc3.plugin
class AutoJoinMessage(AutoJoins):
    requires = ['irc3.plugins.core', ]

    def __init__(self, bot):
        super().__init__(bot)

    def connection_lost(self):
        self.bot.messaging.send_status('DISCONNECTED')
        super(AutoJoinMessage, self).connection_lost()

    def join(self, channel=None):
        super(AutoJoinMessage, self).join(channel)
        self.bot.messaging.send_status('CONNECTED')

    @irc3.event(irc3.rfc.KICK)
    def on_kick(self, mask, channel, target, **kwargs):
        self.bot.messaging.send_status('DISCONNECTED')
        super().on_kick(mask, channel, target, **kwargs)

    @irc3.event("^:\S+ 47[1234567] \S+ (?P<channel>\S+).*")
    def on_err_join(self, channel, **kwargs):
        self.bot.messaging.send_status('DISCONNECTED')
        super().on_err_join(channel, **kwargs)


@irc3.plugin
class EchoToMessage(object):
    requires = ['irc3.plugins.core',
                'irc3.plugins.command']

    def __init__(self, bot):
        self.bot = bot

    @irc3.event(irc3.rfc.PRIVMSG)
    def message(self, mask, event, target, data):
        nick = mask.split('!')[0]
        message = data
        self.bot.messaging.send_message(author=nick, message=str(message))


def create_irc_bot(nick,
                   password,
                   host=None,
                   port=6667,
                   realname=None,
                   channel=None):

    config = dict(ssl=False,
                  includes=['irc3.plugins.core',
                            'irc3.plugins.command',
                            'irc3.plugins.human',
                            __name__])

    if realname is None:
        realname = nick
    if channel is None:
        channel = nick

    config['nick'] = nick
    config['password'] = password
    config['host'] = host
    config['port'] = port
    config['username'] = realname
    config['autojoins'] = channel
    config['level'] = 30

    bot = irc3.IrcBot.from_config(config)

    return bot


async def _check_subscription(bot):
    while True:
        await asyncio.sleep(1)
        msg = None
        try:
            msg = bot.messaging.sub_socket.recv_multipart(zmq.NOBLOCK)
        except zmq.error.Again:
            pass
        if msg:
            msg = decode_vex_message(msg)
            if msg.type == 'CMD':
                bot.command_parser.parse_commands(msg)

            msg = None


def _default(*args, **kwargs):
    pass


def _call_func_with_arg(func, arg):
    def inner(*args):
        return func(arg)
    return inner


def _send_disconnected(messaging):
    def inner():
        messaging.send_status('DISCONNECTED')
    return inner


def _handle_close(messaging):
    def inner(signum=None, frame=None):
        _send_disconnected(messaging)()
    return inner


def main(nick,
         password,
         host,
         channel,
         publish_address,
         subscribe_address,
         service_name):

    irc_client = create_irc_bot(nick,
                                password,
                                host,
                                channel=channel)

    try:
        messaging = ZmqMessaging(service_name,
                                 publish_address,
                                 subscribe_address,
                                 socket_filter='irc')

        messaging.start_messaging()
    except ZMQError:
        return
    # Duck type messaging onto irc_client, FTW
    irc_client.messaging = messaging
    command_parser = AdapterCommandManager(messaging)
    irc_client.command_parser = command_parser

    """
    command_parser.register_command('server config', _default)
    command_parser.register_command('ip', _default)
    command_parser.register_command('join', _default)
    command_parser.register_command('kick', _default)
    command_parser.register_command('part', _default)
    command_parser.register_command('invite', _default)
    command_parser.register_command('topic', _default)
    command_parser.register_command('away', _default)
    """

    irc_client.create_connection()
    irc_client.add_signal_handlers()
    event_loop = asyncio.get_event_loop()
    asyncio.ensure_future(_check_subscription(irc_client))
    atexit.register(_send_disconnected(messaging))

    handle_close = _handle_close(messaging)
    signal.signal(signal.SIGINT, handle_close)
    signal.signal(signal.SIGTERM, handle_close)
    try:
        event_loop.run_forever()
    except KeyboardInterrupt:
        pass
    event_loop.close()
    sys.exit()


def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--nick')
    parser.add_argument('--password')
    parser.add_argument('--host')
    parser.add_argument('--channel')
    parser.add_argument('--publish_address')
    parser.add_argument('--subscribe_address')
    parser.add_argument('--service_name')

    return parser.parse_args()


if __name__ == '__main__':

    kwargs = vars(_get_args())
    main(**kwargs)



import zmq
import zmq.devices

from vexmessage import create_vex_message


class Messaging:
    def __init__(self, settings, context=None):
        self._service_name = b'robot'
        context = context or zmq.Context()

        self._proxy = zmq.devices.ThreadProxy(zmq.XSUB, zmq.XPUB)
        proxy_address = settings.get('proxy_address',
                                     'tcp://127.0.0.1:4002')

        subscribe_address = settings.get('subscribe_address',
                                         'tcp://127.0.0.1:4000')

        publish_address = settings.get('publish_address',
                                       'tcp://127.0.0.1:4001')

        self._proxy.bind_in(publish_address)
        self._proxy.bind_out(subscribe_address)
        self._proxy.bind_mon(proxy_address)

        self.subscription_socket = context.socket(zmq.SUB)
        self.subscription_socket.setsockopt(zmq.SUBSCRIBE, b'')
        self.subscription_socket.connect(subscribe_address)

        self._proxy.start()
        self.publish_socket = context.socket(zmq.PUB)
        self.publish_socket.connect(publish_address)

    def _create_frame(self, type, target='', **contents):
        return create_vex_message(target, 'robot', type, **contents)

    def send_message(self, target='', **msg):
        frame = self._create_frame('MSG', target=target, **msg)
        self.publish_socket.send_multipart(frame)

    def send_command(self, target='', **cmd):
        frame = self._create_frame('CMD', target=target, **cmd)
        self.publish_socket.send_multipart(frame)

    def send_response(self, target, original, **rsp):
        frame = self._create_frame('RSP',
                                   target=target,
                                   original=original,
                                   **rsp)

        self.publish_socket.send_multipart(frame)



import sys
import os
import argparse
import tempfile
import atexit
import signal
from time import sleep
from threading import Thread

import httplib2

from apiclient.discovery import build
from oauth2client.client import flow_from_clientsecrets
from oauth2client.file import Storage
from oauth2client.tools import run_flow, argparser

from zmq import ZMQError
from vexmessage import decode_vex_message

from vexbot.command_managers import AdapterCommandManager
from vexbot.adapters.messaging import ZmqMessaging # flake8: noqa


def _run(messaging):
    command_manager = AdapterCommandManager(messaging)
    while True:
        frame = messaging.sub_socket.recv_multipart()
        message = decode_vex_message(frame)
        if message.type == 'CMD':
            command_manager.parse_commands(message)


def _handle_close(messaging):
    def inner(*args):
        _send_disconnect(messaging)()
    return inner


def _send_disconnect(messaging):
    def inner():
        messaging.send_status('DISCONNECTED')
    return inner


def main(client_secret_filepath, publish_address, subscribe_address):
    try:
        messaging = ZmqMessaging('youtube',
                                 publish_address,
                                 subscribe_address,
                                 'youtube')

        messaging.start_messaging()
    except ZMQError:
        return

    thread = Thread(target=_run, args=(messaging, ))
    thread.daemon = True
    thread.start()
    handle_close = _handle_close(messaging)
    signal.signal(signal.SIGINT, handle_close)
    signal.signal(signal.SIGTERM, handle_close)
    atexit.register(_send_disconnect(messaging))

    scope = ['https://www.googleapis.com/auth/youtube',
             'https://www.googleapis.com/auth/youtube.force-ssl',
             'https://www.googleapis.com/auth/youtube.readonly']

    youtube_api = _youtube_authentication(client_secret_filepath, scope)
    parts = 'snippet'
    livestream_response = youtube_api.liveBroadcasts().list(mine=True,
                                                            part=parts,
                                                            maxResults=1).execute()

    live_chat_id = livestream_response.get('items')[0]['snippet']['liveChatId']

    livechat_response = youtube_api.liveChatMessages().list(liveChatId=live_chat_id, part='snippet').execute()

    next_token = livechat_response.get('nextPageToken')
    polling_interval = livechat_response.get('pollingIntervalMillis')
    polling_interval = _convert_to_seconds(polling_interval)
    messaging.send_status('CONNECTED')

    while True:
        sleep(polling_interval)
        part = 'snippet, authorDetails'
        livechat_response = youtube_api.liveChatMessages().list(liveChatId=live_chat_id, part=part, pageToken=next_token).execute()

        next_token = livechat_response.get('nextPageToken')
        polling_interval = livechat_response.get('pollingIntervalMillis')
        polling_interval = _convert_to_seconds(polling_interval)

        for live_chat_message in livechat_response.get('items'):
            snippet = live_chat_message['snippet']
            if not bool(snippet['hasDisplayContent']):
                continue
            message = snippet['displayMessage']
            author = live_chat_message['authorDetails']['displayName']
            messaging.send_message(author=author, message=message)

    messaging.send_status('DISCONNECTED')

def _convert_to_seconds(milliseconds: str):
    return float(milliseconds)/1000.0


def _get_kwargs():
    parser = argparse.ArgumentParser()
    parser.add_argument('--client_secret_filepath')
    parser.add_argument('--publish_address')
    parser.add_argument('--subscribe_address')
    args = parser.parse_args()
    kwargs = vars(args)

    return kwargs


_YOUTUBE_API_SERVICE_NAME = 'youtube'
_YOUTUBE_API_VERSION = 'v3'
_READ_ONLY = "https://www.googleapis.com/auth/youtube.readonly"


def _youtube_authentication(client_filepath, youtube_scope=_READ_ONLY):
    missing_client_message = "You need to populate the client_secrets.json!"
    flow = flow_from_clientsecrets(client_filepath,
                                   scope=youtube_scope,
                                   message=missing_client_message)

    dir = os.path.abspath(__file__) 
    filepath = "{}-oauth2.json".format(dir)
    # remove old oauth files to be safe
    if os.path.isfile(filepath):
        os.remove(filepath)

    storage = Storage(filepath)
    credentials = storage.get()
    if credentials is None or credentials.invalid:
        credentials = run_flow(flow, storage, argparser.parse_args())
        return build(_YOUTUBE_API_SERVICE_NAME,
                     _YOUTUBE_API_VERSION,
                     http=credentials.authorize(httplib2.Http()))


def _get_youtube_link(client_secrets_filepath):
    youtube_api = youtube_authentication(client_secrets_filepath)
    parts = 'id, snippet, status'
    livestream_requests = youtube_api.liveBroadcasts().list(mine=True,
                                                            part=parts,
                                                            maxResults=5)

    while livestream_requests:
        response = livestream_requests.execute()
        # TODO: add better parsing here
        youtube_id = response.get('items', [])[0]['id']
        return 'http://youtube.com/watch?v={}'.format(youtube_id)


if __name__ == '__main__':
    kwargs = _get_kwargs()
    # OAuth2 lib has some argparse functionality that conflicts with ours
    # delete ours for ease of programming
    for _ in range(4):
        del sys.argv[1]
    main(**kwargs)



import unittest

from vexbot.command_managers import CommandManager


class TestCommandManager(unittest.TestCase):
    def setUp(self):
        messaging = None
        self.command_manager = CommandManager(messaging)

    def test_get_callback_recursively_command_none(self):
        """
        since the command is recursive, can expect to pass in `None` as an
        argument
        """
        result = self.command_manager._get_callback_recursively(None)
        self.assertIsNone(result[0])
        self.assertIsNone(result[1])
        self.assertIsNone(result[2])

    def test_get_callback_recursively_callback_none(self):
        """
        want to make sure that if don't pass in a callback, that we get a
        command

        set a command 'test' with a value of 'blue' and check to see that if we
        pass in that command, we get that value back
        """
        def _test():
            pass

        self.command_manager._commands['test'] = _test

        get_callback = self.command_manager._get_callback_recursively

        callback, command, args = get_callback('test')
        self.assertEqual(callback, _test)
        self.assertEqual(command, 'test')

    def test_get_callback_recursively_callback_is_dict(self):
        """
        since this is a recursive command, want to make sure that that works
        """
        self.command_manager.register_command('test', 'blue')
        get_callback = self.command_manager._get_callback_recursively

        with self.assertRaises(TypeError):
            get_callback('test', ['trigger', ])
        # self.assertEqual(callback, 'blue')

    def test_get_callback_args(self):
        def _test():
            pass
        self.command_manager.register_command('subprocess',
                                              {'settings': _test})

        command = 'subprocess'
        args = ['settings', 'irc']
        get_callback = self.command_manager._get_callback_recursively
        callback, command_string, args = get_callback(command, args)
        self.assertEqual(args[0], 'irc')

if __name__ == '__main__':
    unittest.main()



import os
import random
import string
import tempfile
from subprocess import call


def call_editor(directory=None):
    editor = os.environ.get('EDITOR', 'vim')
    initial_message = b""
    file = None
    filename = None

    if directory is not None:
        # create a random filename
        filename = ''.join(random.choice(string.ascii_uppercase +
                                         string.digits) for _ in range(8))

        filename = filename + '.py'
        # TODO: loop back if isfile is True
        filepath = os.path.join(directory, filename)
        file = open(filepath, 'w+b')
    else:
        file = tempfile.NamedTemporaryFile(prefix=directory, suffix='.py')
        filename = file.name
    current_dir = os.getcwd()
    os.chdir(directory)
    file.write(initial_message)
    file.flush()
    call([editor, filename])
    file.close()
    file = open(filename)
    message = file.read()
    file.close()
    os.chdir(current_dir)

    return message



import os
import re

from setuptools import find_packages, setup

VERSIONFILE = 'vexbot/_version.py'
verstrline = open(VERSIONFILE, 'rt').read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in {}".format(VERSIONFILE))


# directory = os.path.abspath(os.path.dirname(__file__))
"""
with open(os.path.join(directory, 'README.rst')) as f:
    long_description = f.read()
"""

setup(
    name="vexbot",
    version=verstr,
    description='Python personal assistant',
    # long_description=long_description,
    url='https://github.com/benhoff/vexbot',
    license='GPL3',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Operating System :: OS Independent'],
    author='Ben Hoff',
    author_email='beohoff@gmail.com',
    entry_points={'vexbot.adapters': ['shell = vexbot.adapters.shell',
                                      'irc = vexbot.adapters.irc',
                                      'xmpp = vexbot.adapters.xmpp',
                                      'socket_io = vexbot.adapters.socket_io',
                                      'youtube = vexbot.adapters.youtube_api'],

                  'console_scripts': ['vexbot=vexbot.__main__:main']},

    packages= find_packages(), # exclude=['docs', 'tests']
    install_requires=[
        'pluginmanager',
        'pyzmq',
        'vexmessage',
        ],

    extras_require={
        'dev': ['flake8', 'twine'],
        'speechtotext': ['speechtotext'],
        'gui': ['chatimusmaximus'],
        'javascript_webscrapper': ['selenium'],
        'irc': ['irc3'],
        'socket_io': ['requests', 'websocket-client'],
        'xmpp': ['sleekxmpp', 'dnspython3'],
        'youtube': ['google-api-python-client'],
        'microphone': ['microphone'],
        'process_name': ['setproctitle'],
        'database': ['vexstorage'],
    }
)



import argparse
import json
import requests
import html
import logging
import signal
import atexit
from time import sleep
from threading import Thread

import websocket
from zmq import ZMQError
from vexmessage import decode_vex_message

from vexbot.command_managers import AdapterCommandManager
from vexbot.adapters.messaging import ZmqMessaging


class ReadOnlyWebSocket(websocket.WebSocketApp):
    # NOTE: chat_signal defined in `__init__`

    def __init__(self,
                 streamer_name,
                 namespace,
                 website_url,
                 publish_address,
                 subscribe_address,
                 service_name):

        self.log = logging.getLogger(__name__)
        self.log.setLevel(0)
        self.messaging = ZmqMessaging(service_name,
                                      publish_address,
                                      subscribe_address,
                                      'socket_io')

        self.messaging.start_messaging()
        self._streamer_name = streamer_name
        self.namespace = namespace
        self._website_url = website_url
        self.log.info('Getting Socket IO key!')
        self.key, heartbeat = self._connect_to_server_helper()
        self.log.info('Socket IO key got!')
        self.command_manager = AdapterCommandManager(self.messaging)
        self._thread = Thread(target=self.handle_subscription)
        self._thread.start()

        # alters URL to be more websocket...ie
        self._website_socket = self._website_url.replace('http', 'ws')
        self._website_socket += 'websocket/'
        super().__init__(self._website_socket + self.key,
                         on_open=self.on_open,
                         on_close=self.on_close,
                         on_message=self.on_message,
                         on_error=self.on_error)

    def handle_subscription(self):
        while True:
            frame = self.messaging.sub_socket.recv_multipart()
            message = decode_vex_message(frame)
            if message.type == 'CMD':
                self.command_manager.parse_commands(message)

    def repeat_run_forever(self):
        while True:
            try:
                self.run_forever()
            except KeyboardInterrupt:
                break
            except Exception as e:
                self.log.info('Socket IO errors: {}'.format(e))
            self.messaging.send_status('DISCONNECTED')
            sleep(3)
            key, _ = self._connect_to_server_helper()
            self.url = self._website_socket + key

    def _connect_to_server_helper(self):
        r = requests.post(self._website_url)
        params = r.text

        # unused variables are connection_timeout and supported_formats
        key, heartbeat_timeout, _, _ = params.split(':')
        heartbeat_timeout = int(heartbeat_timeout)
        return key, heartbeat_timeout

    def on_open(self, *args):
        logging.info('Websocket open!')

    def on_close(self, *args):
        logging.info('Websocket closed :(')

    def on_message(self, *args):
        message = args[1].split(':', 3)
        key = int(message[0])
        # namespace = message[2]

        if len(message) >= 4:
            data = message[3]
        else:
            data = ''
        if key == 1 and args[1] == '1::':
            self.send_packet_helper(1)
        elif key == 1 and args[1] == '1::{}'.format(self.namespace):
            self.send_packet_helper(5, data={'name': 'initialize'})
            data = {'name': 'join',
                    'args': ['{}'.format(self._streamer_name)]}

            self.send_packet_helper(5, data=data)
            self.log.info('Connected to channel with socket io!')
            self.messaging.send_status('CONNECTED')
        elif key == 2:
            self.send_packet_helper(2)
        elif key == 5:
            data = json.loads(data, )
            if data['name'] == 'message':
                message = data['args'][0]
                sender = html.unescape(message['sender'])
                message = html.unescape(message['text'])
                self.messaging.send_message(author=sender, message=message)

    def on_error(self, *args):
        print(args[1])

    def disconnect(self):
        callback = ''
        data = ''
        # '1::namespace'
        self.send(':'.join([str(self.TYPE_KEYS['DISCONNECT']),
                           callback, self.namespace, data]))

    def send_packet_helper(self,
                           type_key,
                           data=None):

        if data is None:
            data = ''
        else:
            data = json.dumps(data)

        # NOTE: callbacks currently not implemented
        callback = ''
        message = ':'.join([str(type_key), callback, self.namespace, data])
        self.send(message)


def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--streamer_name')
    parser.add_argument('--namespace')
    parser.add_argument('--website_url')
    parser.add_argument('--service_name')
    parser.add_argument('--publish_address')
    parser.add_argument('--subscribe_address')

    return parser.parse_args()


def _handle_close(messaging):
    def inner(*args):
        messaging.send_status('DISCONNECTED')
    return inner


def _send_disconnect(messaging):
    def inner():
        _handle_close(messaging)()
    return inner


if __name__ == '__main__':

    args = vars(_get_args())
    already_running = False
    try:
        client = ReadOnlyWebSocket(**args)
    except ZMQError:
        already_running = True

    if not already_running:
        messaging = client.messaging
        atexit.register(_send_disconnect(messaging))
        handle_close = _handle_close(messaging)
        signal.signal(signal.SIGINT, handle_close)
        signal.signal(signal.SIGTERM, handle_close)
        client.repeat_run_forever()
